@@ -4,6 +4,7 @@ import os
 import sys
 import time
 from datetime import date
+from typing import Union
 
 import gitlab
 from dateutil.relativedelta import relativedelta
@@ -30,10 +31,21 @@ class EnvironementConfig:
         base_env_vars = dict(base_env_vars or {}, **os.environ)
         self.NESTOR_NAME = base_env_vars.get("NESTOR_NAME") or base_env_vars.get("CI_COMMIT_REF_SLUG")
         self.NESTOR_NAME_PREFIX = base_env_vars.get("NESTOR_NAME_PREFIX")
-        self.ALWAYS_DELETE = base_env_vars.get("ALWAYS_DELETE")
+        self.ALWAYS_DELETE: bool = base_env_vars.get("ALWAYS_DELETE", str(False)).capitalize() == str(True).capitalize()
+        self.NEVER_DELETE_ON_FAIL: bool = (
+            base_env_vars.get("NEVER_DELETE_ON_FAIL", str(False)).capitalize() == str(True).capitalize()
+        )
+        self.ALWAYS_RESTORE: bool = (
+            base_env_vars.get("ALWAYS_RESTORE", str(False)).capitalize() == str(True).capitalize()
+        )
+        self.NO_RESET_PASSWORDS: bool = (
+            base_env_vars.get("NO_RESET_PASSWORDS", str(False)).capitalize() == str(True).capitalize()
+        )
         self.CI_COMMIT_REF_NAME = base_env_vars.get("CI_COMMIT_REF_NAME")
         self.CI_PROJECT_DIR = base_env_vars.get("CI_PROJECT_DIR")
+        self.CI_BUILDS_DIR = base_env_vars.get("CI_BUILDS_DIR")
         self.CI_PROJECT_PATH = base_env_vars.get("CI_PROJECT_PATH")
+        self.GITLAB_SERVER_URL = base_env_vars.get("CI_SERVER_URL")
         self.GITLAB_TOKEN = base_env_vars.get("GITLAB_TOKEN")
         self.ODOO_VERSION = base_env_vars.get("ODOO_VERSION")
         self.S3_SECRET_PREVIEW = base_env_vars.get("S3_SECRET_PREVIEW")
@@ -42,18 +54,18 @@ class EnvironementConfig:
         self.VERBOSE = bool(base_env_vars.get("VERBOSE"))
         self.DATE = base_env_vars.get("DATE")
         self.S3_BUCKET = base_env_vars.get("S3_BUCKET")
+        self.ENABLE_QUEUE_JOB = base_env_vars.get("ENABLE_QUEUE_JOB")
         self.PROD_INSTANCE_NAME = base_env_vars.get("PROD_INSTANCE_NAME") or base_env_vars.get("CI_PROJECT_NAME")
         self.DB_NAME = base_env_vars.get("DB_NAME")
         self.ODOO_DEPENDS = base_env_vars.get("ODOO_DEPENDS")
         self.DUMP_PATH = base_env_vars.get("PATH_DUMP")
         self.NO_DB_DUMP: bool = base_env_vars.get("NO_DB_DUMP", str(False)).capitalize() == str(True).capitalize()
         self.MODULES_WITHOUT_DEMO: str = base_env_vars.get("MODULES_WITHOUT_DEMO", str(False).capitalize())
-        self.NESTOR_CFG_FILENAME = os.environ.get("NESTOR_CFG_FILENAME", ".nestor-cfg.yml")
-        self.BASE_MODULE_UPDATE = os.environ.get("BASE_MODULE_UPDATE", "all")
-        self.MODULE_TO_INSTALL = os.environ.get("MODULE_TO_INSTALL")
+        self.NESTOR_CFG_FILENAME = base_env_vars.get("NESTOR_CFG_FILENAME", ".nestor-cfg.yml")
+        self.BASE_MODULE_UPDATE = base_env_vars.get("BASE_MODULE_UPDATE", "all")
+        self.MODULE_TO_INSTALL = base_env_vars.get("MODULE_TO_INSTALL")
 
     def apply_default(self):
-
         self.CI_PROJECT_PATH = self.CI_PROJECT_PATH or CURRENT_PATH
         self.S3_SECRET_PREVIEW = self.S3_SECRET_PREVIEW or "s3-ndp-preview"
         self.S3_DUMP_SECRET = self.S3_DUMP_SECRET or "s3-ndp-scaleway"
@@ -87,7 +99,7 @@ class EnvironementConfig:
         self.DATE = self.DATE or date.today().isoformat()
 
         if self.NESTOR_NAME_PREFIX:
-            self.NESTOR_NAME = f"{self.NESTOR_NAME_PREFIX}-{self.NESTOR_NAME}"
+            self.NESTOR_NAME = f"{self.NESTOR_NAME_PREFIX}{self.NESTOR_NAME}"
 
     def __repr__(self):
         res = "NESTOR_NAME = " + str(self.NESTOR_NAME)
@@ -110,6 +122,11 @@ class EnvironementConfig:
         res += "\nODOO_DEPENDS = " + str(self.ODOO_DEPENDS)
         res += "\nNESTOR_CFG_FILENAME = " + str(self.NESTOR_CFG_FILENAME)
         res += "\nBASE_MODULE_UPDATE = " + str(self.BASE_MODULE_UPDATE)
+        res += "\nNO_RESET_PASSWORDS = " + str(self.NO_RESET_PASSWORDS)
+        res += "\nALWAYS_DELETE = " + str(self.ALWAYS_DELETE)
+        res += "\nALWAYS_RESTORE = " + str(self.ALWAYS_RESTORE)
+        res += "\nNEVER_DELETE_ON_FAIL = " + str(self.NEVER_DELETE_ON_FAIL)
+
         return res
 
 
@@ -117,9 +134,9 @@ class InteractiveConfig(EnvironementConfig):
     def __init__(self):
         super(InteractiveConfig, self).__init__()
         log("Nom du projet git (Sans le 'odoo-addons/' ex: mettre `aef` pour `odoo-addons/aef`) ?")
-        log("Mettre un '/' si c'est un path absolut (ex /odoo/v15/tms/core')")
-        self.CI_PROJECT_PATH = input("=> ")
-        DEFAULT_PROD_INSTANCE_NAME = self.CI_PROJECT_PATH
+        log(f"Mettre un '/' si c'est un path absolut (ex /odoo/v15/tms/core') Defaut: {self.CI_PROJECT_PATH}")
+        self.CI_PROJECT_PATH = input("=> ") or self.CI_PROJECT_PATH
+        DEFAULT_PROD_INSTANCE_NAME = self.PROD_INSTANCE_NAME or self.CI_PROJECT_PATH
         if self.CI_PROJECT_PATH:
             if not self.CI_PROJECT_PATH.startswith("/"):
                 self.CI_PROJECT_PATH = "odoo-addons/" + self.CI_PROJECT_PATH
@@ -129,20 +146,20 @@ class InteractiveConfig(EnvironementConfig):
             log("Un projet est obligatoire")
             sys.exit(1)
 
-        log("Version d'Odoo ?")
-        self.ODOO_VERSION = input("=> ")
+        log("Version d'Odoo ? default ", self.ODOO_VERSION)
+        self.ODOO_VERSION = input("=> ") or self.ODOO_VERSION
 
-        log(f"Nom de la branche ou du tag || default = {self.ODOO_VERSION}")
-        self.CI_COMMIT_REF_NAME = input("=> ") or self.ODOO_VERSION
+        log(f"Nom de la branche ou du tag || default = {self.CI_COMMIT_REF_NAME  or self.ODOO_VERSION}")
+        self.CI_COMMIT_REF_NAME = input("=> ") or self.CI_COMMIT_REF_NAME or self.ODOO_VERSION
 
-        log(f"Nom du nestor (Vide pour utliser '{self.CI_COMMIT_REF_NAME}')")
-        self.NESTOR_NAME = input("=> ") or self.CI_COMMIT_REF_NAME
+        log(f"Nom du nestor (Vide pour utliser '{self.NESTOR_NAME or self.CI_COMMIT_REF_NAME}')")
+        self.NESTOR_NAME = input("=> ") or self.NESTOR_NAME or self.CI_COMMIT_REF_NAME
 
         log(f"Nom de l'instance nestor dans le kube de prod || default '{DEFAULT_PROD_INSTANCE_NAME}'")
         self.PROD_INSTANCE_NAME = input("=> ") or DEFAULT_PROD_INSTANCE_NAME
 
-        log(f"Repertoire du projet ou laisser vide pour utiliser '{CURRENT_PATH}'")
-        self.CI_PROJECT_DIR = input("=> ") or CURRENT_PATH
+        log(f"Repertoire du projet ou laisser vide pour utiliser '{CURRENT_PATH}' default={self.CI_PROJECT_DIR}")
+        self.CI_PROJECT_DIR = input("=> ") or self.CI_PROJECT_DIR or CURRENT_PATH
 
         log(f"Prefix à utliser pour le nom sinon '{self.NESTOR_NAME}'")
         self.NESTOR_NAME_PREFIX = input("=> ") or ""
@@ -152,7 +169,7 @@ class InteractiveConfig(EnvironementConfig):
 
         if not self.NO_DB_DUMP:
             log(f"Nom de la base de backup ? (Vide pour le nom de la prod '{self.PROD_INSTANCE_NAME}')")
-            self.DB_NAME = input("=> ")
+            self.DB_NAME = input("=> ") or self.PROD_INSTANCE_NAME
 
             log(f"Date du dump à utiliser (format ISO YYYY-MM-DD) || default = {date.today().isoformat()}")
             self.DATE = input("=> ") or self.DATE
@@ -222,6 +239,7 @@ class PreviewUtils:
         return nestor_values
 
     def get_spec_values(self, sections: collections.abc.Iterable = ("default",), stage="runtime") -> NestorDescSet:
+        """Retrieve default value from pynestor configuration file and project's configuration file"""
         cfg_file_name = ".nestor-cfg.yml"
         preview_confs_path = os.path.join(os.path.dirname(__file__), "default-preview-nestor-cfg.yml")
         nestor_cfg_path = os.path.join(self.env_config.CI_PROJECT_DIR, cfg_file_name)
@@ -250,14 +268,13 @@ class PreviewUtils:
 class ScriptNestor:
     def __init__(self, config: EnvironementConfig):
         self.config = config
-        self.inst = ScriptNestorInstance(config.NESTOR_NAME)
-        self.gitlab_api = gitlab.Gitlab("https://gitlab.ndp-systemes.fr", config.GITLAB_TOKEN, api_version="4")
+        self.inst = ScriptNestorInstance(config.NESTOR_NAME, allowed_to_delete=not self.config.NEVER_DELETE_ON_FAIL)
+        self.gitlab_api = gitlab.Gitlab(config.GITLAB_SERVER_URL, config.GITLAB_TOKEN, api_version="4")
         self.preview_utils = PreviewUtils(self.gitlab_api, self.config)
 
     def delete(self):
         self.inst.delete()
-        self.inst.wait(up=False, postgres=True)
-        self.inst.wait(up=False)
+        self.wait_all_down_after_delete()
 
     def edit_with_values(self, values):
         log("MAJ de l'instance", self.inst.name)
@@ -279,15 +296,16 @@ class ScriptNestor:
         self.inst.delete_and_exit_if_failed(self.inst.wait(postgres=True, timeout=5 * 60))
         self.inst.delete_and_exit_if_failed(self.inst.wait(timeout=5 * 60))
 
-    def wait_all_down(self):
-        self.inst.delete_and_exit_if_failed(self.inst.wait(up=False, postgres=True, timeout=5 * 60))
-        self.inst.delete_and_exit_if_failed(self.inst.wait(up=False, timeout=5 * 60))
+    def wait_all_down_after_delete(self):
+        """Attend que la BDD et odoo soit down sans planter quand la BDD ets déjà supprimée"""
+        self.inst.wait(up=False, postgres=True, timeout=5 * 60)
+        self.inst.wait(up=False, timeout=5 * 60)
 
     def stop(self):
         self.inst.delete_and_exit_if_failed(self.inst.stop())
         self.inst.delete_and_exit_if_failed(self.inst.wait(up=False, postgres=False, timeout=5 * 60))  # Timeout de 5min
 
-    def enable_s3_spec(self, values):
+    def enable_s3_spec(self, values) -> NestorDescSet:
         if self.inst.version(values) not in SUPPORTED_S3:
             log("Version", self.inst.version(values), "don't support S3 in preview: allowed are", SUPPORTED_S3)
             return values
@@ -302,6 +320,12 @@ class ScriptNestor:
             client.make_bucket(self.config.S3_BUCKET)
         return values
 
+    def enable_queue_job(self, values: NestorDescSet = None) -> NestorDescSet:
+        channels = values["options.queueJobs.channels"] or NestorOpt("options.queueJobs.channels", "root:1")
+        values.add(channels)
+        values.add(NestorOpt("options.queueJobs.enabled", True))
+        return values
+
     def log_spec(self, spec_values: NestorDescSet):
         log("")
         log("Spec Nestor", self.inst.name)
@@ -341,7 +365,8 @@ class PreviewUpScript(ScriptNestor):
                 self.config.ALT_DUMP_PATH,
                 s3_secret=self.config.S3_DUMP_SECRET,
                 bucket=self.config.S3_DUMP_BUCKET,
-                set_password_to_all=True,
+                set_password_to_all=not self.config.NO_RESET_PASSWORDS,
+                no_reset_password=self.config.NO_RESET_PASSWORDS,
             )
         )
 
@@ -351,10 +376,42 @@ class PreviewUpScript(ScriptNestor):
         )
         values.add(self.preview_utils._create_all_spec_sources(self.config.CI_PROJECT_PATH))
         values = self.enable_s3_spec(values)
+        if self.config.ENABLE_QUEUE_JOB:
+            values = self.enable_queue_job(values)
         if self.config.NO_DB_DUMP:
             values = self.add_with_demo_option(values)
+        if self._is_container(values.get("spec.dockerImage", "")):
+            # container image supports only https git fetching
+            values = self._change_git_method_to_https(values)
         return values
 
+    def _change_git_method_to_https(self, values: NestorDescSet) -> NestorDescSet:
+        """Change the git fetch method to https"""
+        for idx in list(range(1, 20)):
+            idx = "" if idx == 1 else idx
+            opt = "spec.sources.repositories.DEPOT_GIT%s" % idx
+            if not values.has_sub_key(opt):
+                break
+            values.add(
+                NestorGitDesc(
+                    idx,
+                    values.get(opt + ".path").value,
+                    values.get(opt + ".branch").value,
+                    method="https",
+                    secret="ndp-gitlab-pull",
+                    server="",
+                )
+            )
+        return values
+
+    def _is_container(self, option: Union[NestorOpt, str]) -> bool:
+        """
+        cornercut method to identify image of type  registry.ndp-systemes.fr/odoo-cloud/container
+        """
+        if isinstance(option, NestorOpt):
+            option = option.value
+        return option.endswith("container")
+
     def run_script(self):
         if not self.inst.name:
             log("Instance don't have a name")
@@ -362,24 +419,30 @@ class PreviewUpScript(ScriptNestor):
         if self.config.ALWAYS_DELETE:
             if self.inst.exist():
                 self.delete()
-                self.wait_all_down()
-
-        if not self.inst.exist():
+                # it seems that the instance is not really deleted event if nestor says it is, so we wait a little
+                time.sleep(30)
+        instance_doesnt_exist = not self.inst.exist()
+        perform_restore = not self.config.NO_DB_DUMP and (self.config.ALWAYS_RESTORE or instance_doesnt_exist)
+        if instance_doesnt_exist:
             log("Create instance")
             values = self.get_spec_values(stage="restore")
             self.log_spec(values)
             self.create_with_values(values)
-            if not self.config.NO_DB_DUMP:
-                self.restore_db()
-            if self.config.MODULE_TO_INSTALL:
-                log("Install -i", self.config.MODULE_TO_INSTALL)
-                self.inst.install(self.config.MODULE_TO_INSTALL)
+        if perform_restore:
+            values = self.get_spec_values(stage="restore")
+            self.edit_with_values(values)
+            self.inst.wait(up=True, postgres=True)
+            self.restore_db()
+        if self.config.MODULE_TO_INSTALL:
+            log("Install -i", self.config.MODULE_TO_INSTALL)
+            self.inst.install(self.config.MODULE_TO_INSTALL)
 
         log("MAJ de l'instance", self.inst.name)
         self.stop()
         values = self.get_spec_values(stage="runtime")
         self.log_spec(values)
         self.edit_with_values(values)
+        self.stop()  # On force un stop d'Odoo au cas ou il a restart pour pas bloquer l'étape d'apres
         log("Attente du demarrage postgresql")
         self.inst.delete_and_exit_if_failed(self.inst.wait(up=True, postgres=True))
         time.sleep(10)
@@ -387,7 +450,6 @@ class PreviewUpScript(ScriptNestor):
         self.inst.delete_and_exit_if_failed(self.inst.update(self.config.BASE_MODULE_UPDATE))
         self.inst.delete_and_exit_if_failed(self.inst.wait(up=True, postgres=True))
         self.inst.delete_and_exit_if_failed(self.inst.start())
-
         self.wait_all_up()
 
         log("=======================================================")
@@ -398,7 +460,8 @@ class PreviewUpScript(ScriptNestor):
         return 0
 
     def add_with_demo_option(self, values):
-        return values.merge(NestorDescSet([NestorOpt("options.withoutDemo", self.config.MODULES_WITHOUT_DEMO)]))
+        without_value = self.config.MODULES_WITHOUT_DEMO.lower() == "all" and True or self.config.MODULES_WITHOUT_DEMO
+        return values.merge(NestorDescSet([NestorOpt("options.withoutDemo", without_value)]))
 
 
 class PreviewDownScript(ScriptNestor):
