@@ -4,7 +4,7 @@ import re
 import subprocess
 import sys
 from functools import cached_property
-from typing import List, Union
+from typing import List, Optional, Set, Union
 
 from yaml import load
 
@@ -46,91 +46,8 @@ class Utils:
         return result
 
 
-class NestorDescSet:
-    def __init__(self, init: List["NestorDesc"] = None):
-        self.values = set()
-        self.add(init or [])
-
-    def add(self, other: Union[List["NestorDescSet"], "NestorDescSet", "NestorDesc", List["NestorDesc"]]) -> None:
-        if not isinstance(other, list):
-            other = [other]
-        for vo in other:
-            if isinstance(vo, NestorOpt):
-                values = [vo]
-            elif isinstance(vo, NestorDesc):
-                values = vo.to_set_opt().values
-            elif isinstance(vo, NestorDescSet):
-                values = vo.values
-            else:
-                raise ValueError("Can't add type %s" % type(vo))
-            for v in values:
-                if v in self.values:
-                    self.values.remove(v)
-                self.values.add(v)
-
-    def merge(self, other: Union["NestorDescSet", List["NestorDescSet"]]):
-        if not isinstance(other, list):
-            other = [other]
-        for vo in other:
-            if not isinstance(vo, NestorDescSet):
-                raise ValueError("Can't add type %s" % type(vo))
-            self.values |= vo.values
-        return self
-
-    def __getitem__(self, item):
-        to_find = item
-        if isinstance(to_find, str):
-            # __hash__ only on the key not on th evalue, TODO replace with cutsom lib internal class
-            to_find = NestorOpt(item, 0)
-
-        for el in self.values:
-            if el == to_find:
-                return el
-        return None
-
-    def get(self, item, default=None):
-        res = self.__getitem__(item)
-        if not res:
-            if callable(default):
-                return default()
-            return default
-        return res
-
-    def __repr__(self):
-        return self.values.__repr__()
-
-    def __str__(self):
-        return self.values.__str__()
-
-    @staticmethod
-    def parse_nestor_cfg(path_nestor_cfg, section):
-        nestor_values = NestorDescSet()
-        stream = open(path_nestor_cfg)
-        data = load(stream, Loader=Loader)
-        print("Parsing section [%s] of %s" % (section, path_nestor_cfg))
-        for key, value in Utils.flat_dico((), data.get(section, {})):
-            try:
-                nestor_values.add(NestorOpt(".".join(key), value))
-            except TypeError as e:
-                print(key, value)
-                print(e, flush=True)
-                raise e
-        return nestor_values
-
-    def to_str(self, pretty=False):
-        prefix = pretty and "\n" or ","
-        return prefix.join(
-            sorted(
-                map(
-                    lambda it: str(it),
-                    self.values,
-                )
-            )
-        )
-
-
 class NestorDesc:
-    def to_set_opt(self) -> NestorDescSet:
+    def to_set_opt(self) -> "NestorDescSet":
         """Return a list of valid parameters for the --set arg for the 'nestor new' command"""
         raise NotImplementedError()
 
@@ -177,7 +94,7 @@ class NestorOpt(NestorDesc):
         self.value = value
 
     def to_set_opt(self):
-        """Return a list off valid paramter for the --set arg for the 'nestor new' command"""
+        """Return a list of valid parameters for the --set arg for the 'nestor new' command"""
         return NestorDescSet([self])
 
     def _key_hash(self):
@@ -200,6 +117,98 @@ class NestorOpt(NestorDesc):
         )
 
 
+class NestorDescSet:
+    """Représente un ensemble de NestorOpt (qui sont des paires clef / valeur)"""
+
+    def __init__(self, init: List["NestorDesc"] = None):
+        self.values: Set[NestorOpt] = set()
+        self.add(init or [])
+
+    def add(self, other: Union[List["NestorDescSet"], "NestorDescSet", "NestorDesc", List["NestorDesc"]]) -> None:
+        if not isinstance(other, list):
+            other = [other]
+        for vo in other:
+            if isinstance(vo, NestorOpt):
+                values = [vo]
+            elif isinstance(vo, NestorDesc):
+                values = vo.to_set_opt().values
+            elif isinstance(vo, NestorDescSet):
+                values = vo.values
+            else:
+                raise ValueError("Can't add type %s" % type(vo))
+            for v in values:
+                if v in self.values:
+                    self.values.remove(v)
+                self.values.add(v)
+
+    def merge(self, other: Union["NestorDescSet", List["NestorDescSet"]]):
+        if not isinstance(other, list):
+            other = [other]
+        for vo in other:
+            if not isinstance(vo, NestorDescSet):
+                raise ValueError("Can't add type %s" % type(vo))
+            self.values |= vo.values
+        return self
+
+    def __getitem__(self, item: Union[str, "NestorDesc"]) -> Optional[NestorOpt]:
+        to_find = item
+        if isinstance(to_find, str):
+            # __hash__ only on the key not on the value, TODO replace with custom lib internal class
+            to_find = NestorOpt(item, 0)
+
+        for el in self.values:
+            if el == to_find:
+                return el
+        return None
+
+    def get(self, item, default=None):
+        res = self.__getitem__(item)
+        if not res:
+            if callable(default):
+                return default()
+            return default
+        return res
+
+    def has_sub_key(self, key: str) -> bool:
+        """
+        True if the set has a key which is a sub key (for example spec.sources.repositories
+        is a subset of spec.sources
+        """
+        return any(k.name.startswith(key) for k in self.values)
+
+    def __repr__(self):
+        return self.values.__repr__()
+
+    def __str__(self):
+        return self.values.__str__()
+
+    @staticmethod
+    def parse_nestor_cfg(path_nestor_cfg, section):
+        nestor_values = NestorDescSet()
+        stream = open(path_nestor_cfg)
+        data = load(stream, Loader=Loader)
+        print("Parsing section [%s] of %s" % (section, path_nestor_cfg))
+        for key, value in Utils.flat_dico((), data.get(section, {})):
+            try:
+                nestor_values.add(NestorOpt(".".join(key), value))
+            except TypeError as e:
+                print(key, value)
+                print(e, flush=True)
+                raise e
+        return nestor_values
+
+    def to_str(self, pretty=False):
+        prefix = pretty and "\n" or ","
+        return prefix.join(
+            sorted(
+                map(
+                    lambda it: str(it),
+                    self.values,
+                )
+            )
+        )
+
+
 class NestorGitDesc(NestorDesc):
     """Python class to handle the yml part of the Odoo operator
     branch: ""
@@ -283,7 +292,7 @@ class NestorCommand:
 
 
 class NestorInstance:
-    def __init__(self, name, verbose=False):
+    def __init__(self, name: str, verbose=False):
         self.name = name
         self.verbose = verbose
         self.filestore = NestorFileStoreCommand(self)
@@ -292,14 +301,16 @@ class NestorInstance:
     def __repr__(self):
         return "nestor[%s]" % self.name
 
-    def version(self, values: NestorDescSet = None):
+    def version(self, values: Optional[NestorDescSet] = None) -> int:
         if self.exist():
             value = self.spec["version"]
         elif values:
             value = values["version"] and values["version"].value
         else:
             raise ValueError("Version can't be found")
-        return int(float(value))
+        # la version peut être 14, 14.0 ou build-14.0-ba5d8624 dans le cas d'une image pas encore déployée
+        numeric_part = value.split("-")[-2:][0]  # on extrait la partie après le - s'il y en a un
+        return int(float(numeric_part))
 
     def save_spec(self, values: NestorDescSet, path_dir: str):
         full_path = path_dir
@@ -375,15 +386,27 @@ class NestorInstance:
         s3_secret: str = None,
         bucket: str = None,
         set_password_to_all: bool = False,
+        no_reset_password: bool = False,
         verbose: bool = False,
     ) -> int:
-
         if self.db.restore(
-            NestorDatabaseCmd.DBCmdTarget.S3, dump_path, s3_secret, bucket, set_password_to_all, verbose
+            NestorDatabaseCmd.DBCmdTarget.S3,
+            dump_path,
+            s3_secret,
+            bucket,
+            set_password_to_all,
+            no_reset_password,
+            verbose,
         ):
             print(f"{dump_path} not found, defaulting to {alt_dump_path}")
             return self.db.restore(
-                NestorDatabaseCmd.DBCmdTarget.S3, alt_dump_path, s3_secret, bucket, set_password_to_all, verbose
+                NestorDatabaseCmd.DBCmdTarget.S3,
+                alt_dump_path,
+                s3_secret,
+                bucket,
+                set_password_to_all,
+                no_reset_password,
+                verbose,
             )
         return 0
 
@@ -430,6 +453,7 @@ class NestorInstance:
         return self._call("install", *opt)
 
     def delete_if_failed(self, result: int) -> bool:
+        """delete when the result code is not zero (meaning that previous operation has failed)"""
         print("Delete ?", bool(result), flush=True)
         if result:
             return bool(self.delete())
@@ -507,6 +531,7 @@ class NestorDatabaseCmd:
         s3_secret: str = None,
         bucket: str = None,
         set_password_to_all: bool = False,
+        no_reset_password: bool = False,
         verbose: bool = False,
     ) -> int:
         if isinstance(type, NestorDatabaseCmd.DBCmdTarget):
@@ -520,6 +545,8 @@ class NestorDatabaseCmd:
             opt.append(bucket)
         if set_password_to_all:
             opt.append("--set-password-to-all")
+        if no_reset_password:
+            opt.append("--no-reset-passwords")
         if verbose:
             opt.append("--verbose")
         return NestorCommand(self._inst, ["database", "restore"], *opt).call()
@@ -613,14 +640,19 @@ class NestorFileStoreCommand:
 
 
 class ScriptNestorInstance(NestorInstance):
-    def __init__(self, name, verbose=False):
+    def __init__(self, name, verbose=False, allowed_to_delete: bool = True):
         super(ScriptNestorInstance, self).__init__(name, verbose)
+        self.allowed_to_delete = allowed_to_delete
 
     @staticmethod
     def from_inst(nestor_inst: NestorInstance):
         return ScriptNestorInstance(nestor_inst.name, nestor_inst.verbose)
 
     def delete_and_exit_if_failed(self, result: int):
-        self.delete_if_failed(result)
+        if self.allowed_to_delete:
+            self.delete_if_failed(result)
+        self.exit_if_failed(result)
+
+    def exit_if_failed(self, result: int):
         if result:
             sys.exit(result)
